<?php

namespace AustraliaPost\Extensions\Business\API;

use AustraliaPost\Core\Api\Abstract_Calculator;
use AustraliaPost\Extensions\Business\API\Wrapper\Account;
use AustraliaPost\Extensions\Business\API\Wrapper\Connector;
use AustraliaPost\Extensions\Business\API\Wrapper\Connectors\Connectors_Factory;
use AustraliaPost\Extensions\Business\API\Wrapper\Connectors\My_Post_Connector;
use AustraliaPost\Extensions\Business\API\Wrapper\Create_Order_Request;
use AustraliaPost\Extensions\Business\API\Wrapper\Create_Pickup_Request;
use AustraliaPost\Extensions\Business\API\Wrapper\Label;
use AustraliaPost\Extensions\Business\API\Wrapper\Organisations;
use AustraliaPost\Extensions\Business\API\Wrapper\Pay_For_Order_Request;
use AustraliaPost\Extensions\Business\API\Wrapper\Price;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Feature;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Item;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Request;
use AustraliaPost\Extensions\Business\API\Wrapper\Shipment_Price;
use AustraliaPost\Extensions\Business\API\Wrapper\Shipment_Price_Request;
use AustraliaPost\Extensions\Business\API\Wrapper\Shipment_Request;
use AustraliaPost\Extensions\Business\Helpers\Utilities;
use AustraliaPost\Helpers\Utilities as RegularUtilities;

class Business extends Abstract_Calculator {
	/**
	 * @var Connector
	 */
	private $connector;
	private $settings = [];

	private $servicesGroups = [
			'regular' => ['B30', 'B31', 'BE1P05', 'BE1P10', 'BE1P30', 'BE1P50',],
			'express' => ['B20', 'B21', 'BE9P05', 'BE9P10', 'BE9P30', 'BE9P50',],
		];


	/**
	 * business constructor.
	 *
	 * @param array $settings
	 */
	public function __construct( $settings = [] ) {
		$this->connector = Utilities::get_api_connector();
		$this->settings  = $settings;
		parent::__construct( $settings );
	}

	/**
	 * @param Shipment_Request $createShipmentRequest
	 *
	 * @return array
	 */
	public function create_shipment( $createShipmentRequest ) {
		return $this->connector->connect( '/shipments', 'POST', Utilities::_json_encode( $createShipmentRequest->getPayload() ) );
	}

	/**
	 * @param string $order_id
	 *
	 * @return array
	 */
	public function get_order( $order_id ) {
		return $this->connector->connect( '/orders/' . $order_id, 'GET' );
	}

	/**
	 * @param Label $label_request
	 *
	 * @return array
	 */
	public function generate_label( $label_request ) {
		return $this->connector->connect( '/labels', 'POST', Utilities::_json_encode( $label_request->getPayLoad() ) );
	}

	/**
	 * @param string $label_request_id
	 *
	 * @return array
	 */
	public function print_label( $label_request_id ) {
		return $this->connector->connect( '/labels/' . $label_request_id, 'GET' );
	}

	/**
	 * @param $shipment_id
	 *
	 * @return array
	 */
	public function get_tracking_info( $shipment_id ) {
		return $this->connector->connect( '/track?tracking_ids=' . $shipment_id, 'GET' );
	}

	/**
	 * @param array $tracking_ids
	 *
	 * @return array
	 */
	public function get_tracking_events_by_tracking_ids( $tracking_ids ) {
		return $this->connector->connect( '/track?tracking_ids=' . implode( ',', $tracking_ids ), 'GET' );
	}

	public function calculate( $package_details, $package ) {

		$this->debug( 'Packing Details', $package_details );

		if ( $package['destination']['country'] == 'AU' ) {
			$trackedLettersRates = $this->add_tracked_letters_rates( $package_details, [] );
			if ( count( $trackedLettersRates ) > 0 ) {
				return $trackedLettersRates;
			}
		}

		$priceRequest = new Price_Request();
		$priceRequest->setFromPostcode( $this->settings['shop_post_code'] );
		$priceRequest->setFromCountry( 'AU' );
		$priceRequest->setToPostcode( $package['destination']['postcode'] );
		$priceRequest->setToCountry( $package['destination']['country'] );
		foreach ( $package_details as $pack ) {
			$pack      = $this->adjust_dimensions( $pack );
			$priceItem = ( new Price_Item( Utilities::accountType() ) )
				->setLength( $pack['length'] )
				->setWidth( $pack['width'] )
				->setHeight( $pack['height'] )
				->setWeight( $pack['weight'] )
				->setProductIds( $this->getEnabledProducts( $pack, $priceRequest->getToCountry() ) )
				->setItemReference( $pack['item_id'] );

			if ( Utilities::isMyPost() ) {
				if ( $this->settings['signature_on_delivery'] === 'yes' ) {
					$sod_feature = new Price_Feature();
					$sod_feature->setCode( 'SIGNATURE_ON_DELIVERY' );
					$sod_feature->setAttributes( [] );
					$priceItem->addFeature( $sod_feature );
				}
			} else {
				$priceItem->setSignatureOnDelivery( $this->settings['signature_on_delivery'] === 'yes' );
			}

			if ( $this->settings['enable_extra_cover'] === 'yes' ) {
				$extra_cover_feature = new Price_Feature();
				$extra_cover_feature->setCode( 'TRANSIT_COVER' );
				$extra_cover_feature->setAttributes( [ 'cover_amount' => round( floatval( $pack['value'] ), 2 ) ] );
				$priceItem->addFeature( $extra_cover_feature );

				if ( $pack['value'] > 500 && $this->settings['signature_on_delivery'] === 'no' ) {
					$sod_feature = new Price_Feature();
					$sod_feature->setCode( 'SIGNATURE_ON_DELIVERY' );
					$sod_feature->setAttributes( [] );
					$priceItem->addFeature( $sod_feature );
				}
			}

			$priceRequest->addItem( $priceItem );
		}


		if ( $this->calculateWithSurcharges() ) {
			$rates = $this->getPricesWithSurcharges( $priceRequest, $package );
		} else {
			$rates = $this->getPrices( $priceRequest );
		}

		if ( isset( $rates['errors'] ) && count( $rates['errors'] ) > 0 ) {
			$this->debug( 'errors', $rates['errors'], 'error' );
			return null;
		}


		$this->debug( 'rates', $rates );

		return $rates;
	}

	/**
	 * a fix for: "At least 2 dimensions must be 5 cm" error.
	 *
	 * @param array $pack
	 *
	 * @return array
	 */
	private function adjust_dimensions( $pack ) {
		$dimensions = [
			'length' => $pack['length'],
			'width'  => $pack['width'],
			'height' => $pack['height'],
		];

		$max_dimension = array_search( max( $dimensions ), $dimensions );

		foreach ( $dimensions as $key => $dimension ) {
			if ( $key === $max_dimension ) {
				continue;
			}
			if ( $dimension < 5 ) {
				$dimensions[ $key ] = 5;
			}
		}

		$pack['length'] = $dimensions['length'];
		$pack['width']  = $dimensions['width'];
		$pack['height'] = $dimensions['height'];

		return $pack;
	}

	/**
	 * @param $weight
	 * @param $country
	 *
	 * @return array
	 */
	private function getEnabledProducts( $params, $country ) {
		if ( $this->settings['account_type'] !== 'mypost' ) {
			return [];
		}

		if ( $country === 'AU' ) {
			return $this->getEnabledDomesticProducts( $params );
		}

		$products             = [];
		$enabled_intl_options = $this->settings['intl_options'];

		if ( $params['weight'] > 2 ) {
			if ( in_array( 'INT_PARCEL_STD_OWN_PACKAGING', $enabled_intl_options ) ) {
				$products[] = 'I65';
			}
			if ( in_array( 'INT_PARCEL_EXP_OWN_PACKAGING', $enabled_intl_options ) ) {
				$products[] = 'I67';
			}

			return $products;
		}

		if ( in_array( 'INT_PARCEL_SEA_OWN_PACKAGING', $enabled_intl_options ) || in_array( 'INT_PARCEL_AIR_OWN_PACKAGING', $enabled_intl_options ) ) {
			$products[] = 'I63';
		}

		if ( in_array( 'INT_PARCEL_STD_OWN_PACKAGING', $enabled_intl_options ) ) {
			$products[] = 'I64';
		}

		if ( in_array( 'INT_PARCEL_EXP_OWN_PACKAGING', $enabled_intl_options ) ) {
			$products[] = 'I67';
		}

		return $products;

	}

	private function getEnabledDomesticProducts( $params ) {

		$products        = [];
		$cubicWeightInKg = ( ( $params['length'] * 0.01 ) * ( $params['width'] * 0.01 ) * ( $params['height'] * 0.01 ) ) * 250;

		if ($params['weight'] > 1 && max( $params['weight'], $cubicWeightInKg ) > 5 ) {
			if ( in_array( 'AUS_PARCEL_REGULAR', $this->settings['domestic_options'] ) ) {
				$products[] = 'B31';
			}
			if ( in_array( 'AUS_PARCEL_EXPRESS', $this->settings['domestic_options'] ) ) {
				$products[] = 'B21';
			}

			return $products;
		}

		if ( is_array( $this->settings['satchels'] ) && count( $this->settings['satchels'] ) === 0 ) {
			if ( in_array( 'AUS_PARCEL_REGULAR', $this->settings['domestic_options'] ) ) {
				$products[] = 'B30';
			}
			if ( in_array( 'AUS_PARCEL_EXPRESS', $this->settings['domestic_options'] ) ) {
				$products[] = 'B20';
			}

			return $products;
		}

		$satchelCodes = [
			'regular' => [
				'small'  => 'BE1P05',
				'1kg'    => 'BE1P10',
				'medium' => 'BE1P30',
				'large'  => 'BE1P50',
			],
			'express' => [
				'small'  => 'BE9P05',
				'1kg'    => 'BE9P10',
				'medium' => 'BE9P30',
				'large'  => 'BE9P50',
			],
		];


		if ( in_array( 'AUS_PARCEL_REGULAR', $this->settings['domestic_options'] ) ) {
			foreach ( $satchelCodes['regular'] as $size => $code ) {
				if ( isset( $this->settings['satchels']['regular'][ $size ] ) && RegularUtilities::fit_satchel_size( $params, $size ) ) {
					$products[] = $code;
					break;
				}
			}
		}

		if ( in_array( 'AUS_PARCEL_EXPRESS', $this->settings['domestic_options'] ) ) {
			foreach ( $satchelCodes['express'] as $size => $code ) {
				if ( isset( $this->settings['satchels']['express'][ $size ] ) && RegularUtilities::fit_satchel_size( $params, $size ) ) {
					$products[] = $code;
					break;
				}
			}
		}

		return $products;
	}

	/**
	 * @param Price_Request $priceRequest
	 * @param array $package
	 *
	 * @return array
	 */
	private function getPricesWithSurcharges( $priceRequest, $package ) {

		$priceRequest->setFromState( WC()->countries->get_base_state() )
		             ->setFromSuburb( WC()->countries->get_base_city() )
		             ->setToState( $package['destination']['state'] )
		             ->setToSuburb( $package['destination']['city'] );
		$products = Utilities::get_method_setting('enabled_contract_products');

		if( empty($products) ) {
			return [];
		}

		$filtered_products = $this->filter_domestic_intl_products($products, $package['destination']['country']);

		$shipmentPricesRequest = new Shipment_Price_Request();
		/** @var Price_Item $item */
		foreach ($filtered_products as $product) {


			$shipment = new Price_Request();
			foreach ( $priceRequest->getItems() as $item ) {
				$newItem = clone $item;
				$newItem->setProductId($product);
				$shipment->addItem($newItem);
			}

			$shipment->setFromPostcode( $this->settings['shop_post_code'] )
			         ->setFromCountry( WC()->countries->get_base_country() )
			         ->setToPostcode( $package['destination']['postcode'] )
			         ->setToCountry( $package['destination']['country'] )
			         ->setFromState( WC()->countries->get_base_state() )
			         ->setFromSuburb( WC()->countries->get_base_city() )
			         ->setToState( $package['destination']['state'] )
			         ->setToSuburb( $package['destination']['city'] );

			$shipmentPricesRequest->addShipment( $shipment );

		}

		$prices = ( new Shipment_Price( Utilities::get_api_connector(), $shipmentPricesRequest ) )->getItemPrices();

		if ( isset( $prices['errors'] ) ) {
			$this->debug( 'Prices With Surcharges Errors', $prices );
			return [];
		}
		$this->debug('Surcharge', $prices);
		$rates = [];
		foreach ( $prices as $product_id => $price ) {

			WC()->session->set( 'extra_cover_total', null );
			WC()->session->set( 'sod_fee', null );
			$extra_cover_cost = 0;
			if ( $this->settings['seperate_extracover_sod'] === 'yes' ) {
				if ( isset( $price['items'][0]['features']['TRANSIT_COVER'] ) ) {
					$extra_cover_cost = $price['items'][0]['features']['TRANSIT_COVER']['price']['calculated_price'];
					WC()->session->set( 'extra_cover_total', $extra_cover_cost );
				}
			}
			$sod_cost = 0;
			if ( $this->settings['seperate_extracover_sod'] === 'yes' ) {
				if ( isset( $price['items'][0]['features']['SIGNATURE_ON_DELIVERY'] ) ) {
					$sod_cost = $price['items'][0]['features']['SIGNATURE_ON_DELIVERY']['price']['calculated_price'];
					WC()->session->set( 'sod_fee', $sod_cost );
				}
			}

			$cost = round( $price['total_cost'] - ( $extra_cover_cost + $sod_cost ), 2 );

			if ( isset( $rates[ $product_id ] ) ) {
				$cost = $cost + $rates[ $product_id ]['cost'];
			}

			$cost = $this->strip_shipping_tax( $cost );
			$cost = $cost + $this->calculate_handling_fee( $cost );

			$rates[ $product_id ] = [
				'id'    => 'aus:' . $product_id . ':' . $this->settings['instance_id'],
				'label' => $this->get_label( $product_id ),
				'cost'  => $cost
			];
		}

		if ( count( $rates ) === 0 && isset( $prices['errors'] ) ) {
			return $prices;
		}

		return $rates;

	}

	/**
	 * @param string $product_id
	 *
	 * @return bool
	 */
	private function is_product_enabled( $product_id ) {

		// if the option does not exist, let's display all options.
		if ( ! isset( $this->settings['enabled_contract_products'] ) ) {
			return true;
		}

		if ( ! is_array( $this->settings['enabled_contract_products'] ) ) {
			return false;
		}

		return in_array( $product_id, $this->settings['enabled_contract_products'] );
	}

	/**
	 * @param string $product_id
	 * @param string $product_type
	 *
	 * @return string
	 */
	private function get_label( $product_id ) {
		$custom_titles = Utilities::get_method_setting( 'custom_titles' );

		if ( isset( $custom_titles[ $product_id ] ) && trim( $custom_titles[ $product_id ] ) !== '' ) {
			return $custom_titles[ $product_id ];
		}

		return Utilities::get_product_name( $product_id );

	}

	/**
	 * @param Price_Request $priceRequest
	 *
	 * @return array
	 */
	public function getPrices( $priceRequest ) {
		$rates = [];
		$price = new Price( $this->connector, $priceRequest );

		$prices = $price->getItemPrices();
		foreach ( $prices as $price ) {
			if ( ! isset( $price['product_id'] ) ) {
				continue;
			}

			if ( $this->settings['account_type'] === 'eparcel' ) {
				if ( ! $this->is_product_enabled( $price['product_id'] ) ) {
					continue;
				}
			}

			WC()->session->set( 'extra_cover_total', null );
			WC()->session->set( 'sod_fee', null );
			$extra_cover_cost = 0;
			if ( $this->settings['seperate_extracover_sod'] === 'yes' ) {
				if ( isset( $price['features']['TRANSIT_COVER'] ) ) {
					$extra_cover_cost = $price['features']['TRANSIT_COVER']['price']['calculated_price'];
					WC()->session->set( 'extra_cover_total', $extra_cover_cost );
				}
			}
			$sod_cost = 0;
			if ( $this->settings['seperate_extracover_sod'] === 'yes' ) {
				if ( isset( $price['features']['SIGNATURE_ON_DELIVERY'] ) ) {
					$sod_cost = $price['features']['SIGNATURE_ON_DELIVERY']['price']['calculated_price'];
					WC()->session->set( 'sod_fee', $sod_cost );
				}
			}

			$cost = round( $price['calculated_price'] - ( $extra_cover_cost + $sod_cost ), 2 );

			if ( isset( $rates[ $price['product_id'] ] ) ) {
				$cost = $cost + $rates[ $price['product_id'] ]['cost'];
			}

			$cost = $this->strip_shipping_tax( $cost );
			$cost = $cost + $this->calculate_handling_fee( $cost );

			$rates[ $price['product_id'] ] = [
				'id'    => 'aus:' . $price['product_id'] . ':' . $this->settings['instance_id'],
				'label' => $this->get_label( $price['product_id'] ),
				'cost'  => $cost
			];
		}

		if ( count( $rates ) === 0 && isset( $prices['errors'] ) ) {
			return $prices;
		}

		$product_ids = array_keys($rates);

		// group standard and express international together for mypost accounts.
		if (Utilities::isMyPost() && $priceRequest->getToCountry() !== 'AU') {
			if (count(array_intersect(['I64', 'I65'], $product_ids)) == 2) {
				$rates['I65']['cost'] += $rates['I64']['cost'];
				unset($rates['I64']);
			}
			if (count(array_intersect(['I66', 'I67'], $product_ids)) == 2) {
				$rates['I66']['cost'] += $rates['I67']['cost'];
				unset($rates['I67']);
			}
		}

		// group standard and express domestic together for mypost accounts.
		if (Utilities::isMyPost() && $priceRequest->getToCountry() === 'AU') {
			$new_rates = [];
			foreach ($rates as $key => $rate) {
				if (in_array($key, $this->servicesGroups['regular'])) {
					$new_rates['regular']['cost'] += $rate['cost'];
					$new_rates['regular']['id'] = $rate['id'];
					$new_rates['regular']['label'] = $rate['label'];
				}

				if (in_array($key, $this->servicesGroups['express'])) {
					$new_rates['express']['cost'] += $rate['cost'];
					$new_rates['express']['id'] = $rate['id'];
					$new_rates['express']['label'] = $rate['label'];
				}
			}

			if (!in_array( 'AUS_PARCEL_EXPRESS', $this->settings['domestic_options'])) {
				unset($new_rates['express']);
			}

			if (!in_array( 'AUS_PARCEL_REGULAR', $this->settings['domestic_options'])) {
				unset($new_rates['regular']);
			}

			return $new_rates;
		}

		return $rates;
	}

	/**
	 * @param Create_Order_Request $createOrderRequest
	 *
	 * @return array
	 */
	public function create_order( $createOrderRequest ) {
		return $this->connector->connect( '/orders', 'PUT', Utilities::_json_encode( $createOrderRequest->getPayload() ) );
	}

	/**
	 * @param Pay_For_Order_Request $payForOrderRequest
	 *
	 * @return array
	 */
	public function create_payment( $payForOrderRequest ) {
		return $this->connector->connect( '/payments', 'POST', Utilities::_json_encode( $payForOrderRequest->getPayload() ) );
	}

	/**
	 * @param Create_Pickup_Request $createPickupRequest
	 *
	 * @return array
	 */
	public function create_pickup( $createPickupRequest ) {
		return $this->connector->connect( '/pickups', 'POST', Utilities::_json_encode( $createPickupRequest->getPayload() ) );
	}

	/**
	 * @param string $account_number
	 * @param string $order_id
	 *
	 * @return array
	 */
	public function download_order_summary( $account_number, $order_id ) {
		return $this->connector->connect( sprintf( '/accounts/%s/orders/%s/summary', $account_number, $order_id ), 'GET' );
	}

	/**
	 * @param string $accountNumber
	 * @param string $accountPassword
	 * @param string $apiKey
	 *
	 * @return boolean
	 */
	public function verify_credentials( $accountNumber, $accountPassword, $apiKey ) {
		$connector = Connectors_Factory::make( 'eparcel', [
			'username'       => $apiKey,
			'password'       => $accountPassword,
			'account_number' => $accountNumber,
		] );

		$response = ( new Account( $connector ) )->get_account();

		if ( isset( $response['account_number'] ) ) {
			return $response['account_number'] === $accountNumber;
		}

		return false;
	}

	/**
	 * @param string $merchantToken
	 *
	 * @return array
	 */
	public function get_account_by_merchant_token( $merchantToken ) {
		/** @var My_Post_Connector $connector */
		$connector = Connectors_Factory::make( 'mypost', [
			'merchant_token' => $merchantToken,
			'account_number' => null,
		] );

		return ( new Organisations( $connector ) )->get_account();
	}

	private function calculateWithSurcharges() {
		return Utilities::isEParcel() && Utilities::get_method_setting( 'calculate_with_surcharges' ) === 'yes';
	}

	private function filter_domestic_intl_products( $products, $country ) {

		$filtered_products = [];
		foreach ($products as $product) {
			if($country !== 'AU') {
				if($this->is_product_intl($product)) {
					$filtered_products[] = $product;
				}
			} else {
				if(!$this->is_product_intl($product)) {
					$filtered_products[] = $product;
				}

			}


		}

		return $filtered_products;
	}

	private function is_product_intl($product) {
		if (substr( $product, 0, 3 ) === "RPI") {
			return true;
		}
		if (substr( $product, 0, 3 ) === "PTI") {
			return true;
		}
		if (substr( $product, 0, 2 ) === "IC") {
			return true;
		}
		if (substr( $product, 0, 3 ) === "ECM") {
			return true;
		}
		if (substr( $product, 0, 3 ) === "AIR") {
			return true;
		}

		return false;
	}

}
