<?php

namespace AustraliaPost\Extensions\Business\Endpoints;

use AustraliaPost\Extensions\Business\API\Business;
use AustraliaPost\Extensions\Business\Helpers\Utilities;
use AustraliaPost\Extensions\Business\Settings\Product_Shipping_Fields;
use WC_Order;
use WC_Order_Item_Product;

class Load_App_Data_Endpoint extends Abstract_Endpoint {

	/**
	 * @param $data
	 *
	 * @return array
	 */
	public function callback( $data )
	{
		$order = $this->get_order($data);

		$payload = [];
		$payload = $this->add_defaults($payload, $order);
		$payload = $this->add_shipments($payload, $order);
		$payload = $this->add_labels($payload, $order);
		$payload = $this->add_from_address($payload);
		$payload = $this->add_to_address($payload, $order);
		$payload = $this->add_dangerous_goods_flag($payload, $order);

		return $payload;
	}

	/**
	 * @return string
	 */
	public function action()
	{
		return 'load_app_data';
	}

	private function add_shipments( $payload, $order )
	{

		$shipments = json_decode($order->get_meta(Create_Shipments_Endpoint::LABELS_SHIPMENTS_META_KEY), true);
		$payload['shipments'] = [];

		if (!is_array($shipments)) {
			return $payload;
		}

		$tracking_ids = [];
		$shipment_ids = [];

		foreach ($shipments as $key => $shipment) {
			$item = $shipment['items'][0];
			$shipment_ids[] = $shipment['shipment_id'];
			$payload['shipments'][$key]['tracking_events'] = [];
			$payload['shipments'][$key]['weight'] = $item['weight'] * 1000; // kg to g
			$payload['shipments'][$key]['product_id'] = $item['product_id'];
			$payload['shipments'][$key]['tracking_details'] = $item['tracking_details'];

			if (isset($item['tracking_details']['article_id'])) {
				$tracking_ids[$key] = $item['tracking_details']['article_id'];
			}

			$payload['shipments'][$key]['price']['calculated_gst'] = $shipment['shipment_summary']['total_gst'];
			$payload['shipments'][$key]['price']['calculated_price'] = $shipment['shipment_summary']['total_cost'];
			$payload['shipments'][$key]['price']['calculated_price_ex_gst'] = $shipment['shipment_summary']['total_cost_ex_gst'];
			$payload['shipments'][$key]['price']['product_type'] = Utilities::get_product_name($item['product_id']);

			$payload['shipments'][$key]['label_details']['shipment_id'] = $shipment['shipment_id'];
			$payload['shipments'][$key]['label_details']['item_id'] = $item['item_id'];

			$payload['shipments'][$key]['clearance_items'] = [];
		}

		$tracking_data = $this->get_tracking_events($tracking_ids);
		foreach ($tracking_data as $shipment_key => $tracking_events) {
			if (!isset($tracking_events['trackable_items'])) {
				continue;
			}

			if (count($tracking_events['trackable_items']) === 0) {
				continue;
			}



			$events = [];
			$status = '';

			if (isset($tracking_events['trackable_items'][0]['events']) && count($tracking_events['trackable_items'][0]['events']) > 0) {
				$events = $tracking_events['trackable_items'][0]['events'];
			} elseif (isset($tracking_events['trackable_items'][0]['items']) && isset($tracking_events['trackable_items'][0]['items']['events']) && count($tracking_events['trackable_items'][0]['items']['events']) > 0) {
				$events = $tracking_events['trackable_items'][0]['items']['events'];
			}

			if(isset($tracking_events['status'])) {
				$status = $tracking_events['status'];
			}

			$payload['shipments'][$shipment_key]['tracking_events'] = $events;
			$payload['shipments'][$shipment_key]['tracking_status'] = $status;

		}

		$payload['shipments_ordered'] = $this->shipmentsOrdered($shipment_ids);

		return $payload;
	}

	private function add_labels( $payload, $order )
	{
		$labels = json_decode($order->get_meta(Generate_Labels_Endpoint::LABELS_LABELS_META_KEY), true);

		if ($labels) {
			$payload['label_request_id'] = $labels['labels'][0]['request_id'];
			$payload['label_url'] = '';
		}

		return $payload;
	}

	/**
	 * @param array $payload
	 *
	 * @return array
	 */
	private function add_from_address( $payload )
	{
		$payload['from_address'] = $this->get_from_address()->getPayLoad();
		return $payload;
	}

	/**
	 * @param array $payload
	 * @param \WC_Order $order
	 *
	 * @return array
	 */
	private function add_to_address( $payload, $order )
	{
		$payload['to_address'] = $this->get_to_address($order)->getPayLoad();
		return $payload;
	}

	/**
	 * @param array $tracking_ids
	 *
	 * @return array
	 */
	private function get_tracking_events($tracking_ids)
	{
		if (count($tracking_ids) === 0) {
			return [];
		}

		$response = (new Business())->get_tracking_events_by_tracking_ids($tracking_ids);
		if (isset($response['tracking_results'])) {
			return $response['tracking_results'];
		}

		return [];
	}

	/**
	 * @param array $payload
	 * @param WC_Order $order
	 * @return array
	 */
	private function add_defaults( $payload, $order )
	{
		$default_size = Utilities::get_method_setting('default_size');
		$packages = json_decode($order->get_meta( WPRUBY_PACKAGING_DETAILS_KEY ), true);
		$payload['packages'] = [];

		if ($packages) {
			foreach ($packages as $package) {
				$payload['packages'][] = [
					'weight' => $package['weight'] * 1000,
					'length' => (isset($package['real_length']) ? $package['real_length']: $package['length']) * 10,
					'width' => (isset($package['real_width']) ? $package['real_width']: $package['width']) * 10,
					'height' => (isset($package['real_height']) ? $package['real_height']: $package['height']) * 10,
				];
			}
			return $payload;
		}

		$order_items = $order->get_items();
		$weight = 0;
		/** @var WC_Order_Item_Product $item */
		foreach ($order_items as $item) {
			$quantity = $item->get_quantity();
			$product = $item->get_product();
			while ($quantity !== 0){
				$weight += wc_get_weight($product->get_weight(), 'g');
				$quantity--;
			}
		}

		if (count($default_size) === 0) {
			$default_size = ['length'=> 100, 'width'=>100, 'height'=>100];
		} else {
			$default_size = array_map([$this, 'to_mm'], $default_size);
		}

		$payload['defaults'] = [];
		$payload['defaults']['dimensions'] = $default_size;
		$payload['defaults']['weight'] = $weight;

		$payload['order_subtotal'] = $order->get_subtotal();
		$payload['order_description'] = $this->get_order_product_description($order);

		return $payload;
	}

	private function to_mm($dimension) {
		if (!$dimension) {
			return 100;
		}
		return wc_get_dimension($dimension, 'mm');
	}

	/**
	 * @param array $shipmentIds
	 *
	 * @return bool
	 */
	private function shipmentsOrdered($shipmentIds)
	{
		$printed_shipments = get_option(Create_Order_From_Shipments_Endpoint::OPTION_KEY_ORDERED_SHIPMENTS, []);
		if (count($printed_shipments) === 0) {
			return false;
		}
		$intersection = array_intersect($shipmentIds, $printed_shipments);

		return count($shipmentIds) === count($intersection);

	}

	/**
	 * @param array $payload
	 * @param WC_Order $order
	 * @return array
	 */
	private function add_dangerous_goods_flag( $payload, $order )
	{
		$payload['has_dangerous_goods'] = false;
		foreach ($order->get_items() as $item)
		{
			$payload['has_dangerous_goods'] = 'yes' === get_post_meta($item->get_data()['product_id'], Product_Shipping_Fields::DANGEROUS_GOODS_KEY, true);
			if ($payload['has_dangerous_goods'] === true) {
				break;
			}
		}

		return $payload;
	}

	/**
	 * @param WC_Order $order
	 */
	private function get_order_product_description( $order )
	{
		/** @var \WC_Order_Item $item */
		$items = array_map(function ($item) {
			return $item->get_name();
		}, $order->get_items());

		return implode(',', $items);
	}

}
