<?php
namespace AustraliaPost\Extensions\Business\Helpers;

use AustraliaPost\Extensions\Business\API\Wrapper\Account;
use AustraliaPost\Extensions\Business\API\Wrapper\Connectors\Abstract_Connector;
use AustraliaPost\Extensions\Business\API\Wrapper\Connectors\Connectors_Factory;
use AustraliaPost\Extensions\Business\API\Wrapper\Connectors\E_Parcel_Connector;
use AustraliaPost\Extensions\Business\API\Wrapper\Connectors\My_Post_Connector;

class Utilities extends \AustraliaPost\Helpers\Utilities {

	const POSTAGE_PRODUCTS_TS_KEY = 'wpruby_australia_post_shipment_postage_products';

	/**
	 * @return Abstract_Connector
	 */
	public static function get_api_connector()
	{
		$account_type = self::get_method_setting('account_type');

		if (!$account_type) {
			$account_type = 'mypost';
		}

		if ($account_type === 'eparcel') {
			$credentials = [
				'username' => self::get_method_setting('api_key'),
				'password' => self::get_method_setting('api_password'),
				'account_number' => self::get_method_setting('api_account_number'),
			];
		} else {
			$credentials = [
				'merchant_token' => self::get_method_setting('api_merchant_token'),
				'account_number' => get_option(My_Post_Connector::ACCOUNT_NUMBER_OPTION_KEY),
			];
		}

		return Connectors_Factory::make($account_type, $credentials);
	}

	/**
	 * @param bool $as_object
	 *
	 * @return array
	 */
	public static function get_postage_products($as_object = false, $account_type = null)
	{
		$account_type = ($account_type !== null)? $account_type: self::get_method_setting('account_type');

		if ($account_type === 'mypost') {
			return self::my_post_products($as_object);
		}

		$postage_products =  get_transient(self::POSTAGE_PRODUCTS_TS_KEY);

		if ( $postage_products === false ) {

			/** @var E_Parcel_Connector $connector */
			$connector = self::get_api_connector();

			$postage_products = (new Account($connector))->get_postage_products();

			if (count($postage_products) > 0) {
				set_transient(self::POSTAGE_PRODUCTS_TS_KEY, $postage_products, 604800); # 2 weeks
			}
		}

		if ($as_object === true) {
			return $postage_products;
		}

		$normalized_products = [];
		foreach ($postage_products as $postage_product) {
			$normalized_products[$postage_product['product_id']] = $postage_product['type'];
		}

		return $normalized_products;
	}

	/**
	 * @return string[]
	 */
	public static function my_post_products_labels()
	{
		return  [
			'B30' => 'Parcel Post',
			'B31' => 'Parcel Post',
			'BE1P05' => 'Parcel Post Satchel (Small)',
			'BE1P10' => 'Parcel Post Satchel (Medium)',
			'BE1P30' => 'Parcel Post Satchel (Large)',
			'BE1P50' => 'Parcel Post Satchel (Extra Large)',
			'BE1PB1' => 'Parcel Post Flat Rate Box (Small)',
			'BE1PB2' => 'Parcel Post Flat Rate Box (Medium)',
			'BE1PB3' => 'Parcel Post Flat Rate Box (Large)',
			'BE1PB4' => 'Parcel Post Flat Rate Box (Extra Large)',
			'B20' => 'Express Post',
			'B21' => 'Express Post',
			'BE9P05' => 'Express Post Satchel (Small)',
			'BE9P10' => 'Express Post Satchel (Medium)',
			'BE9P30' => 'Express Post Satchel (Large)',
			'BE9P50' => 'Express Post Satchel (Extra Large)',
			'BE9PB1' => 'Express Post Flat Rate Box (Small)',
			'BE9PB2' => 'Express Post Flat Rate Box (Medium)',
			'BE9PB3' => 'Express Post Flat Rate Box (Large)',
			'BE9PB4' => 'Express Post Flat Rate Box (Extra Large)',
			'I63' => 'Economy Air',
			'I64' => 'Standard Small',
			'I65' => 'Standard Large',
			'I67' => 'Express',
		];
	}
	/**
	 * @return string[]
	 */
	public static function my_post_products($as_object = false)
	{

		$products = self::my_post_products_labels();

		unset($products['B31']);
		unset($products['B21']);

		if ($as_object === false) {
			return $products;
		}
		$object = [];
		foreach ($products as $key => $product) {
			$object[] = [
				'product_id' => $key,
				'type' => $product,
			];
		}
		return $object;
	}

	public static function accountType()
	{
		return self::get_method_setting('account_type');
	}

	public static function isEParcel()
	{
		return self::get_method_setting('account_type') === 'eparcel';
	}

	public static function isMyPost()
	{
		return self::get_method_setting('account_type') === 'mypost';
	}

	/**
	 * @return string
	 */
	public static function accountNumber()
	{
		if (self::isMyPost()) {
			return get_option(My_Post_Connector::ACCOUNT_NUMBER_OPTION_KEY);
		}

		return Utilities::get_method_setting('api_account_number');
	}

	public static function get_product_name( $product_id ) {

		if ( self::isEParcel()) {
			$products = self::get_postage_products();
			return $products[$product_id];
		}

		$my_post_products = self::my_post_products_labels();
		if (isset($my_post_products[$product_id])) {
			return $my_post_products[$product_id];
		}

		return $product_id;
	}
}
